﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using FliUtils.Jobs;
using System.Windows.Media;
using System.Windows;

namespace FliProCameraLib
{
    /// <summary>
    /// This file contains Frame related properties and methods
    /// for the FliProCamera class
    /// 2016/09/17
    /// </summary>
    public partial class FliProCamera : FliPro
    {
        //**************************************************************
        #region Constants and Enums

        #endregion Constants and Enums

        //**************************************************************
        #region Public Classes

        #endregion Public Classes

        //**************************************************************
        #region Protected Properties
        #endregion Protected Properties

        //**************************************************************
        #region Public Properties
        #endregion Public Properties

        //**************************************************************
        #region Observable Properties

        //-------------------------------------------
        /// <summary>
        /// Most recent camera base temperature
        /// </summary>
        public double AmbientTemperature
        {
            get { return _AmbientTemperature; }
            protected set { SetValue<double>(ref _AmbientTemperature, value); }
        }
        protected double _AmbientTemperature;

        //-------------------------------------------
        /// <summary>
        /// Apply the Channel Averaging algorythm to the image data
        /// </summary>
        public bool ApplyChannelAveraging
        {
            get { return _ApplyChannelAveraging; }
            set
            {
                if (SetValue<bool>(ref _ApplyChannelAveraging, value))
                { 
                    if (value && ((HBin > 1) || (VBin > 1)))
                    {
                        MessageBox.Show("Channel Averaging cannot be applied when Binning is > 1.");
                        _ApplyChannelAveraging = false;
                    }
                }
            }
        }
        private bool _ApplyChannelAveraging = true;

        //-------------------------------------------
        /// <summary>
        /// Start/Stop background flush
        /// </summary>
        public FliBgFlush BackgroundFlush
        {
            get { return _BackgroundFlush; }
            set { SetValue<FliBgFlush>(ref _BackgroundFlush, value); }
        }
        protected FliBgFlush _BackgroundFlush;

        //-------------------------------------------
        /// <summary>
        /// Most recent camera base temperature
        /// </summary>
        public double BaseTemperature
        {
            get { return _BaseTemperature; }
            protected set { SetValue<double>(ref _BaseTemperature, value); }
        }
        protected double _BaseTemperature;

        public uint CoolerDAC
        {
            get { return _CoolerDAC; }
            set
            {
                if (SetValue<uint>(ref _CoolerDAC, value))
                {
                    SetCoolerDAC(value);
                }
            }
        }
        protected uint _CoolerDAC = 214;

        public bool CoolerDutyCycleIsEnabledOff
        {
            get { return _CoolerDutyCycleIsEnabledOff; }
            set
            {
                if (SetValue<bool>(ref _CoolerDutyCycleIsEnabledOff, value))
                {
                    if (CoolerDutyCycleIsEnabledOn != !value)
                    {
                        CoolerDutyCycleIsEnabledOn = !value;
                    }
                }
            }
        }
        protected bool _CoolerDutyCycleIsEnabledOff = true;

        public bool CoolerDutyCycleIsEnabledOn
        {
            get { return _CoolerDutyCycleIsEnabledOn; }
            set
            {
                if (SetValue<bool>(ref _CoolerDutyCycleIsEnabledOn, value))
                {
                    if (CoolerDutyCycleIsEnabledOff != !value)
                    {
                        CoolerDutyCycleIsEnabledOff = !value;
                    }
                    if (value && CoolerTemperatureOn)
                    {
                        CoolerTemperatureOn = false;
                    }
                    SetCoolerDutyCycleEnable(value);
                }
            }
        }
        protected bool _CoolerDutyCycleIsEnabledOn = false;

        public uint CoolerDutyCyclePower
        {
            get { return _CoolerDutyCyclePower; }
            set
            {
                if (value > 100)
                {
                    value = 100;
                }
                if (SetValue<uint>(ref _CoolerDutyCyclePower, value))
                {
                    SetCoolerDutyCyclePower(value);
                }
            }
        }
        protected uint _CoolerDutyCyclePower;

        //-------------------------------------------
        /// <summary>
        /// Enable/Disable Camera Cooler
        /// </summary>
        public bool CoolerEnable
        {
            get { return _CoolerEnable; }
            protected set
            {
                if (SetValue<bool>(ref _CoolerEnable, value))
                {
                    SetCoolerEnable(value);
                }
            }
        }
        protected bool _CoolerEnable;

        //-------------------------------------------
        /// <summary>
        /// Camera cooler power
        /// </summary>
        public uint CoolerPower
        {
            get { return _CoolerPower; }
            protected set
            {
                if (value > 100) value = 100;
                if (SetValue<uint>(ref _CoolerPower, value))
                {
                    SetCoolerPower(value);
                }
            }
        }
        protected uint _CoolerPower;

        ///// <summary>
        ///// The Cooler Temperature Off radio button value
        ///// </summary>
        public bool CoolerTemperatureOff
        {
            get { return _CoolerTemperatureOff; }
            set
            {
                if (SetValue(ref _CoolerTemperatureOff, value))
                {
                    if (CoolerTemperatureOn != !value)
                    {
                        CoolerTemperatureOn = !value;
                    }
                }
            }
        }
        private bool _CoolerTemperatureOff = false;

        ///// <summary>
        ///// The Cooler Temperature On radio button value
        ///// </summary>
        public bool CoolerTemperatureOn
        {
            get { return _CoolerTemperatureOn; }
            set
            {
                if (SetValue(ref _CoolerTemperatureOn, value))
                {
                    if (CoolerTemperatureOff != !value)
                    {
                        CoolerTemperatureOff = !value;
                    }
                    if (value && CoolerDutyCycleIsEnabledOn)
                    {
                        CoolerDutyCycleIsEnabledOn = false;
                    }
                    if (_CoolerTemperatureOn)
                    {
                        // Only send the target temperature if the flag is on
                        SetTargetTemperature(TargetTemperature);
                    }
                    else
                    {
                        // Just set thge temperature high enough so that the controller is off
                        SetTargetTemperature(100);
                    }
                }
            }
        }
        private bool _CoolerTemperatureOn = true;

        public FliGPSState CurrentGPSState
        {
            get { return _CurrentGPSState; }
            set { SetValue<FliGPSState>(ref _CurrentGPSState, value); }
        }
        protected FliGPSState _CurrentGPSState = FliGPSState.FPRO_GPS_NOT_DETECTED;

        //-------------------------------------------
        /// <summary>
        /// Set the DAC value
        /// </summary>
        public uint DAC
        {
            get { return _DAC; }
            set
            {
                if (SetValue<uint>(ref _DAC, value))
                {
                    SetDac(value);
                }
            }
        }
        protected uint _DAC;

        //-------------------------------------------
        /// <summary>
        /// Exposure time in mSec
        /// </summary>
        public int FrameInterval
        {
            get { return _FrameInterval; }
            set
            {
                if (SetValue<int>(ref _FrameInterval, value))
                {
                    SetFrameInterval(value);
                }
            }
        }
        protected int _FrameInterval = 100;

        public bool FanIsEnabledOff
        {
            get { return _FanIsEnabledOff; }
            set
            {
                if (SetValue<bool>(ref _FanIsEnabledOff, value))
                {
                    if (FanIsEnabledOn != !value)
                    {
                        FanIsEnabledOn = !value;
                    }
                }
            }
        }
        protected bool _FanIsEnabledOff = false;

        public bool FanIsEnabledOn
        {
            get { return _FanIsEnabledOn; }
            set
            {
                if (SetValue<bool>(ref _FanIsEnabledOn, value))
                {
                    if (FanIsEnabledOff != !value)
                    {
                        FanIsEnabledOff = !value;
                    }
                    SetFanEnable(value);
                }
            }
        }
        protected bool _FanIsEnabledOn = true;

        public uint FanPower
        {
            get { return _FanPower; }
            set
            {
                if (value > 100)
                {
                    value = 100;
                }
                if (SetValue<uint>(ref _FanPower, value))
                {
                    SetFanPower(value);
                }
            }
        }
        protected uint _FanPower = 85;

        public Visibility GPSConnectedVisibility
        {
            get { return (IsConnectedGPS ? Visibility.Visible : Visibility.Collapsed); }
        }

        public Visibility GPSLockedVisibility
        {
            get { return (CurrentGPSState == FliGPSState.FPRO_GPS_DETECTED_AND_SAT_LOCK ? Visibility.Visible : Visibility.Collapsed); }
        }

        public Visibility GPSUnlockedVisibility
        {
            get { return (CurrentGPSState == FliGPSState.FPRO_GPS_DETECTED_NO_SAT_LOCK ? Visibility.Visible : Visibility.Collapsed); }
        }

        public int GPSWidth
        {
            get { return (IsConnectedGPS ? 25 : 0); }
        }

        public int GPSLockWidth
        {
            get { return (IsConnectedGPS ? (CurrentGPSState == FliGPSState.FPRO_GPS_DETECTED_AND_SAT_LOCK ? 10 : 15) : 0); }
        }

        //-------------------------------------------
        /// <summary>
        /// Enable/Disable Camera Heater
        /// </summary>
        public bool HeaterEnabledOff
        {
            get { return _HeaterEnabledOff; }
            set
            {
                if (SetValue<bool>(ref _HeaterEnabledOff, value))
                {
                    if (HeaterEnabledOn != !value)
                    {
                        HeaterEnabledOn = !value;
                    }
                }
            }
        }
        protected bool _HeaterEnabledOff = true;

        //-------------------------------------------
        /// <summary>
        /// Enable/Disable Camera Heater
        /// </summary>
        public bool HeaterEnabledOn
        {
            get { return _HeaterEnabledOn; }
            set
            {
                if (SetValue<bool>(ref _HeaterEnabledOn, value))
                {
                    if (HeaterEnabledOff != !value)
                    {
                        HeaterEnabledOff = !value;
                    }
                    SetHeaterEnable(value);
                }
            }
        }
        protected bool _HeaterEnabledOn = false;

        //-------------------------------------------
        /// <summary>
        /// Most recent camera heater power
        /// </summary>
        public uint HeaterPower
        {
            get { return _HeaterPower; }
            set
            {
                if (value > 100) value = 100;
                if (SetValue<uint>(ref _HeaterPower, value))
                {
                    SetHeaterPower(value);
                }
            }
        }
        protected uint _HeaterPower;

        public string HistogramAtDefault
        {
            get { return _HistogramAtDefault; }
            set { SetValue<string>(ref _HistogramAtDefault, value); }
        }
        protected string _HistogramAtDefault = "True";

        public Brush HistogramAtDefaultTextColor
        {
            get { return _HistogramAtDefaultTextColor; }
            set { SetValue(ref _HistogramAtDefaultTextColor, value); }
        }
        private Brush _HistogramAtDefaultTextColor = Brushes.Black;

        public bool IlluminationOff
        {
            get { return _IlluminationOff; }
            set
            {
                if (SetValue<bool>(ref _IlluminationOff, value))
                {
                    if (IlluminationOn != !value)
                    {
                        IlluminationOn = !value;
                    }
                }
            }
        }
        protected bool _IlluminationOff = true;

        public bool IlluminationOn
        {
            get { return _IlluminationOn; }
            set
            {
                if (SetValue<bool>(ref _IlluminationOn, value))
                {
                    if (IlluminationOff != !value)
                    {
                        IlluminationOff = !value;
                    }
                    SetIlluminationOn(value);
                }
            }
        }
        protected bool _IlluminationOn = false;

        public int IlluminationOnDelay
        {
            get { return _IlluminationOnDelay; }
            set
            {
                if (SetValue<int>(ref _IlluminationOnDelay, value))
                {
                    SetIlluminationDelay(value, IlluminationOffDelay);
                }
            }
        }
        protected int _IlluminationOnDelay;

        public int IlluminationOffDelay
        {
            get { return _IlluminationOffDelay; }
            set
            {
                if (SetValue<int>(ref _IlluminationOffDelay, value))
                {
                    SetIlluminationDelay(IlluminationOnDelay, value);
                }
            }
        }
        protected int _IlluminationOffDelay;

        /// <summary>
        /// Flag to signal to do immediate image capture without going back to row zero
        /// </summary>
        public bool Immediate
        {
            get { return _Immediate; }
            set
            {
                if (SetValue<bool>(ref _Immediate, value))
                {
                    SetImmediate(value);
                }
            }
        }
        private bool _Immediate = false;

        public bool IsConnectedGPS
        {
            get { return (CurrentGPSState != FliGPSState.FPRO_GPS_NOT_DETECTED); }
        }

        //-------------------------------------------
        /// <summary>
        /// Read IO Port
        /// Note: This property exists to support binding.
        /// It is updated and NotifyPropertyChange is raised by ReadInputPort()
        /// </summary>
        public int LastInputPort
        {
            get { return _LastInputPort; }
            protected set { SetValue<int>(ref _LastInputPort, value); }
        }
        protected int _LastInputPort;

        public bool LEDOff
        {
            get { return _LEDOff; }
            set
            {
                if (SetValue<bool>(ref _LEDOff, value))
                {
                    if (LEDOn != !value)
                    {
                        LEDOn = !value;
                    }
                }
            }
        }
        protected bool _LEDOff = true;

        public bool LEDOn
        {
            get { return _LEDOn; }
            set
            {
                if (SetValue<bool>(ref _LEDOn, value))
                {
                    if (LEDOff != !value)
                    {
                        LEDOff = !value;
                    }
                    SetLEDOn(value);
                }
            }
        }
        protected bool _LEDOn = false;

        public uint LEDOnDuration
        {
            get { return _LEDOnDuration; }
            set
            {
                if (SetValue<uint>(ref _LEDOnDuration, value))
                {
                    SetLEDOnDuration(value);
                }
            }
        }
        protected uint _LEDOnDuration = 0;

        public bool ManualShutterControlEnabled
        {
            get { return _ManualShutterControlEnabled; }
            set
            {
                if (SetValue<bool>(ref _ManualShutterControlEnabled, value))
                {
                    if (value)
                    {
                        SetShutterOpen(ShutterOpen);
                    }
                    else
                    {
                        ShutterOpen = false;
                        SetShutterOverride(false);
                    }
                }
            }
        }
        protected bool _ManualShutterControlEnabled = false;

        public int MaximumFrameStorage
        {
            get { return _MaximumFrameStorage; }
            set
            {
                if (SetValue<int>(ref _MaximumFrameStorage, value))
                {
                    PurgeFrameList(PurgeActionEnum.FinalPurge);
                }
            }
        }
        protected int _MaximumFrameStorage = 10;

        //-------------------------------------------
        /// <summary>
        /// Number of Flush cycles
        /// </summary>
        public int NFlushes
        {
            get { return _NFlushes; }
            set
            {
                if (SetValue<int>(ref _NFlushes, value))
                {
                    SetNFlushes(value);
                }
            }
        }
        protected int _NFlushes;

        //-------------------------------------------
        /// <summary>
        /// I/OPort configuration
        /// </summary>
        public int PortConfig
        {
            get { return _PortConfig; }
            set
            {
                if (SetValue<int>(ref _PortConfig, value))
                {
                    SetPortConfig(value);
                }
            }
        }
        protected int _PortConfig;        

        public bool ReadDuringExposureOff
        {
            get { return _ReadDuringExposureOff; }
            set
            {
                if (SetValue<bool>(ref _ReadDuringExposureOff, value))
                {
                    if (ReadDuringExposureOn != !value)
                    {
                        ReadDuringExposureOn = !value;
                    }
                }
            }
        }
        protected bool _ReadDuringExposureOff = true;

        public bool ReadDuringExposureOn
        {
            get { return _ReadDuringExposureOn; }
            set
            {
                if (SetValue<bool>(ref _ReadDuringExposureOn, value))
                {
                    if (ReadDuringExposureOff != !value)
                    {
                        ReadDuringExposureOff = !value;
                    }
                    SetReadDuringExposure(value);
                    if (value)
                    {
                        if (TemperatureReadInterval > 0)
                        {
                            TemperatureStatusTimer.Interval = TemperatureReadInterval;
                        }
                    }
                    else
                    {
                        TemperatureStatusTimer.Interval = FLI_DEFAULT_TEMPERATURE_READ_INTERVAL;
                    }
                }
            }
        }
        protected bool _ReadDuringExposureOn = false;

        //-------------------------------------------
        /// <summary>
        /// Shutter Control
        /// </summary>
        public FliShutterControl Shutter
        {
            get { return _Shutter; }
            set
            {
                SetValue<FliShutterControl>(ref _Shutter, value);
                SetShutter(value);
                ShutterOpen = (value != FliShutterControl.CLOSE);
            }
        }
        protected FliShutterControl _Shutter;

        public bool ShutterClose
        {
            get { return _ShutterClose; }
            set
            {
                if (SetValue<bool>(ref _ShutterClose, value) )
                {
                    if (ShutterOpen != !value)
                    {
                        ShutterOpen = !value;
                    }
                }
            }
        }
        protected bool _ShutterClose = true;
        
        public bool ShutterOpen
        {
            get { return _ShutterOpen; }
            set
            {
                if (SetValue<bool>(ref _ShutterOpen, value))
                {
                    if (ShutterClose != !value)
                    {
                        ShutterClose = !value;
                    }
                    SetShutterOpen(value);
                }
            }
        }
        protected bool _ShutterOpen = false;

        public int ShutterOpenDelay
        {
            get { return _ShutterOpenDelay; }
            set
            {
                if (SetValue<int>(ref _ShutterOpenDelay, value))
                {
                    SetShutterDelay(value, ShutterCloseDelay);
                }
            }
        }
        protected int _ShutterOpenDelay = 0;

        public int ShutterCloseDelay
        {
            get { return _ShutterCloseDelay; }
            set
            {
                if (SetValue<int>(ref _ShutterCloseDelay, value))
                {
                    SetShutterDelay(ShutterOpenDelay, value);
                }
            }
        }
        protected int _ShutterCloseDelay;

        //-------------------------------------------
        /// <summary>
        /// Target setpoint for base temperature
        /// </summary>
        public double TargetTemperature
        {
            get { return _TargetTemperature; }

            set
            {
                if (SetValue<double>(ref _TargetTemperature, value))
                {
                    SetTemperatureTextColor();
                    if (CoolerTemperatureOn)
                    {
                        SetTargetTemperature(value);
                    }
                }
            }
        }
        protected double _TargetTemperature = 25;

        //-------------------------------------------
        /// <summary>
        /// Camera TDI rate
        /// </summary>
        public int TDIRate
        {
            get { return _TDIRate; }
            set
            {
                if (SetValue<int>(ref _TDIRate, value))
                {
                    SetTDIRate(value);
                }
            }
        }
        protected int _TDIRate;

        //-------------------------------------------
        /// <summary>
        /// Incremented on each status update
        /// </summary>
        public double Temperature
        {
            get { return _Temperature; }
            protected set
            {
                if (SetValue<double>(ref _Temperature, value))
                {
                    SetTemperatureTextColor();
                }
            }
        }
        protected double _Temperature;

        public int TemperatureReadInterval
        {
            get { return _TemperatureReadInterval; }
            set
            {
                if (SetValue<int>(ref _TemperatureReadInterval, value))
                {
                    if (ReadDuringExposureOn)
                    {
                        TemperatureStatusTimer.Interval = TemperatureReadInterval;
                    }
                }
            }
        }
        protected int _TemperatureReadInterval;


        //-------------------------------------------
        /// <summary>
        /// Property to display temperature status in the bottom of the Main Window
        /// </summary>
        public string TemperatureStatusText
        {
            get { return _TemperatureStatusText; }
            set { SetValue(ref _TemperatureStatusText, value); }
        }
        private string _TemperatureStatusText = String.Empty;

        //-------------------------------------------
        /// <summary>
        /// Property to display temperature on the Grab Panel
        /// </summary>
        public string TemperatureText
        {
            get { return _TemperatureText; }
            set { SetValue(ref _TemperatureText, value); }
        }
        private string _TemperatureText = "0.0";

        public Brush TemperatureTextColor
        {
            get { return _TemperatureTextColor; }
            set { SetValue(ref _TemperatureTextColor, value); }
        }
        private Brush _TemperatureTextColor = Brushes.Black;

        //-------------------------------------------
        /// <summary>
        /// Apply the Channel Averaging algorythm to the image data
        /// </summary>
        public bool UseOverscanColumns
        {
            get { return _UseOverscanColumns; }
            set { SetValue<bool>(ref _UseOverscanColumns, value); }
        }
        private bool _UseOverscanColumns = false;

        #endregion Observable Properties

        //**************************************************************
        #region Public Methods

        public virtual bool GetShutterOpen() { return ShutterOpen; }

        public void OpenShutter(bool isOpen)
        {
            Shutter = (isOpen ? FliShutterControl.OPEN : FliShutterControl.CLOSE);
        }

        //-------------------------------------------
        /// <summary>
        /// Read input port and raise NotifyPropertyChanged if needed
        /// </summary>
        /// <returns></returns>
        public virtual int ReadInputPort()
        {
            LastInputPort = 0;
            return LastInputPort;

        }

        public void SetTemperatureTextColor()
        {
            if (CoolerTemperatureOff || ((Temperature > TargetTemperature - 0.2) && (Temperature < TargetTemperature + 0.2)))
            {
                TemperatureTextColor = Brushes.Black;
            }
            else
            {
                TemperatureTextColor = Brushes.Red;
            }
        }

        /// <summary>
        /// Cancel the active exposure, if any
        /// </summary>
        public virtual void CancelExposure()
        {
        }

        public virtual void ClearTriggerWatcher()
        {
        }

        public virtual int GetRemainingExposureTime()
        {
            return 0;
        }

        public virtual void ExposeFrame()
        {
        }

        public virtual void EndExposure()
        {
        }

        public virtual void ForceTraining(FliProCamera camera) { }

        //-------------------------------------------
        /// <summary>
        /// Read camera sensor temperature 
        /// </summary>
        /// <returns></returns>
        public virtual double GetSensorTemperature()
        {
            SensorTemperature = 0;
            return SensorTemperature;
        }

        //-------------------------------------------
        /// <summary>
        /// Read camera base temperature 
        /// </summary>
        /// <returns></returns>
        public virtual double GetBaseTemperature()
        {
            BaseTemperature = 0;
            return BaseTemperature;
        }

        #endregion Public Methods

        //**************************************************************
        #region Protected Methods
        
        protected virtual void SetCoolerDAC(uint value) { }
        protected virtual void SetCoolerDutyCycleEnable(bool value) { }
        protected virtual void SetCoolerDutyCyclePower(uint value) { }
        protected virtual void SetCoolerEnable(bool value) { }
        protected virtual void SetCoolerPower(uint value) { }
        protected virtual void SetDac(uint value) { }
        protected virtual void SetFanEnable(bool value) { }
        protected virtual void SetFanPower(uint value) { }
        protected virtual void SetFrameInterval(int value) { }
        protected virtual void SetHeaterEnable(bool value) { }
        protected virtual void SetHeaterPower(uint value) { }
        protected virtual void SetIlluminationDelay(int ondelay, int offdelay) { }
        protected virtual void SetIlluminationOn(bool on) { }
        protected virtual void SetImmediate(bool value) { }
        protected virtual void SetLEDOn(bool on) { }
        protected virtual void SetLEDOnDuration(uint uiDuration) { }
        protected virtual void SetNFlushes(int value) { }
        protected virtual void SetPortConfig(int value) { }
        protected virtual void SetReadDuringExposure(bool enable) { }
        protected virtual void SetScanDirection(bool hInverted, bool vInverted) { }
        protected virtual void SetShutter(FliShutterControl value) { }
        protected virtual void SetShutterDelay(int ondelay, int offdelay) { }
        public virtual void SetShutterOpen(bool open) { }
        public virtual void SetShutterOverride(bool value) { }
        protected virtual void SetTargetTemperature(double value) { }
        protected virtual void SetTDIRate(int value) { }
        protected virtual void SetThumbnailCapture(bool value) { }
        public virtual void SetTrainingEnable(bool enable) { }
        protected virtual void SetVideoModeEnabled(bool value) { }

        #endregion Protected Methods

    }
}
