﻿//**************************************************************
// AppliedLogix DotNet Utilities Library
// Color to Brush converter for use in XAML Bindings
//
// Copyright(c) 2016-2017 AppliedLogix, LLC
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
//**************************************************************
using System;
using System.Globalization;
using System.Windows;
using System.Windows.Data;
using System.Windows.Media;

//********************************************************************************
// Usage:
// Declare namespace:
//          xmlns:cvt="clr-namespace:MemToolApp.Converters"
// In Binding, specify Converter class name without "Extension". e.g.:":
//           IsEnabled="{Binding xxxx, Converter={cvt:ColorToBrush}}"
//
//********************************************************************************

namespace FliUtils.Converters
{
    //-------------------------------------------
	/// <summary>
	/// Convert a Color to a SolidColorBrush
	/// </summary>
	public class ColorToBrushExtension : ValueConverterBase
	{
        private static ColorToBrushExtension _converter;
        public override object ProvideValue(IServiceProvider serviceProvider)
        {
            if (_converter == null)
            {
                _converter = new ColorToBrushExtension();
            }
            return _converter;
        }

        public override object Convert(object value, Type targetType, object parameter, CultureInfo culture)
		{
            object rtn = DependencyProperty.UnsetValue;
			if (value is Color)
			{
				Color input = (Color) value;
				rtn = new SolidColorBrush(input);
			}
            return rtn;
		}

		public override object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
		{
            object rtn = DependencyProperty.UnsetValue;
			if (value is SolidColorBrush)
			{
				SolidColorBrush input = (SolidColorBrush)value;
				rtn = input.Color;
			}
            return rtn;
		}
	}

    //-------------------------------------------
    /// <summary>
    /// Convert a SolidColorBrush to a color
    /// </summary>
    public class BrushToColorExtension : ValueConverterBase
    {
        private static BrushToColorExtension _converter;
        public override object ProvideValue(IServiceProvider serviceProvider)
        {
            if (_converter == null)
            {
                _converter = new BrushToColorExtension();
            }
            return _converter;
        }

        public override object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            object rtn = DependencyProperty.UnsetValue;
            if (value is SolidColorBrush)
            {
                SolidColorBrush input = (SolidColorBrush)value;
                rtn = input.Color;
            }
            return rtn;
        }

        public override object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            object rtn = DependencyProperty.UnsetValue;
            if (value is Color)
            {
                Color input = (Color)value;
                rtn = new SolidColorBrush(input);
            }
            return rtn;
        }
    }

}
