﻿//**************************************************************
// AppliedLogix DotNet Utilities Library
// Image converters for use in XAML Bindings
//
// Copyright(c) 2016-2017 AppliedLogix, LLC
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
//**************************************************************
using System;
using System.Drawing;
using System.IO;
using System.Windows.Media.Imaging;

//********************************************************************************
// Image converters for use in XAML Bindings
// Usage:
// Declare namespace:
//          xmlns:cvt="clr-namespace:MemToolApp.Converters"
// In Binding, specify Converter class name without "Extension". e.g.:":
//           IsEnabled="{Binding xxxx, Converter={cvt:Invert}}"
//          Visibility="{Binding xxxx, Converter={cvt:TrueVisibleFalseCollapsed}}"
//
//********************************************************************************

namespace FliUtils.Converters
{
    //-------------------------------------------
    /// <summary>
    /// Convert bool to its complement
    /// </summary>
    public class BitmapToImageExtension : ValueConverterBase
    {
        private static BitmapToImageExtension _converter;
        public override object ProvideValue(IServiceProvider serviceProvider)
        {
            if (_converter == null)
            {
                _converter = new BitmapToImageExtension();
            }
            return _converter;
        }

        public override object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            MemoryStream ms = new MemoryStream();
            ((Bitmap)value).Save(ms, System.Drawing.Imaging.ImageFormat.Png);
            BitmapImage image = new BitmapImage();
            image.BeginInit();
            ms.Seek(0, SeekOrigin.Begin);
            image.StreamSource = ms;
            image.EndInit();

            return image;
        }

        public override object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

}
