﻿//**************************************************************
// AppliedLogix DotNet Utilities Library
// Base class for XAML Binding value converters
//
// Copyright(c) 2016-2017 AppliedLogix, LLC
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
//**************************************************************
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Windows.Markup;
using System.Windows.Data;

namespace FliUtils.Converters
{
    [MarkupExtensionReturnType(typeof(IValueConverter))]
    public abstract class ValueConverterBase : MarkupExtension, IValueConverter
    {
        public virtual object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        public virtual object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        protected static string GetString(object value)
        {
            return value.ToString();
        }

        protected static bool GetBool(object value)
        {
            bool val;
            bool.TryParse(value.ToString(), out val);
            return val;
        }

        protected static int GetInt(object value)
        {
            int val;
            int.TryParse(value.ToString(), out val);
            return val;
        }

        protected static double GetLong(object value)
        {
            long val;
            long.TryParse(value.ToString(), out val);
            return val;
        }

        protected static double GetDouble(object value)
        {
            double val = double.NaN;
            if (value != null)
            {
                double.TryParse(value.ToString(), out val);
            }
            return val;
        }
    }
}
